/***************************************************************************//**
 * @file Cashier.cpp
 * @author Dorian Weber
 * @brief Implementation of the Cashier.
 ******************************************************************************/

#include "Simulation.hpp"

Checkout::Cashier::Cashier(Simulation& sim, Register& lane)
:	Process(sim, "Cashier"),
	lane(lane),
	bill(sim, "Bill", 0, 1),
	money(sim, "Money", 0, 1),
	idleTime(sim, "IdleTime"),
	salesVol(sim, "SalesVolume") {
	// add statistical variables to report
	sim.report.addReportProducer(idleTime);
	sim.report.addReportProducer(salesVol);
}

int
Checkout::Cashier::main() {
	std::unique_ptr<Merchandise> item;
	Simulation& sim = static_cast<Simulation&>(getSimulation());
	double sum;

	while (true) {
		// reset the costs for the current customer
		sum = 0;

		while (true) {
			odemx::base::SimTime delta;

			delta = getTime();
			if (!(item = lane.beltGet()))
				break;
			delta = getTime() - delta;
			idleTime.update(delta);
			sim.idleTime.update(delta);

			// transfer from the belt
			holdFor(item->duration);

			// add the price
			sum += item->price;

			// transfer to the tray
			holdFor(item->duration);
			lane.trayPut(item.release());
		}

		// produce the bill
		bill.release(1);

		// wait for the customer to pay
		money.acquire(1);
		salesVol.update(sum);
		sim.salesVol.update(sum);
	}
}
