/***************************************************************************//**
 * @file Cashier.hpp
 * @author Dorian Weber
 * @brief Contains the definition of the Cashier process.
 ******************************************************************************/

#ifndef CASHIER_HPP_INCLUDED
#define CASHIER_HPP_INCLUDED

#include "Simulation.hpp"

namespace Checkout
{
	// forward declaration
	class Register;
	
	/**@brief Active class modeling the cashier occupying the Register.
	 */
	class Cashier: public odemx::base::Process
	{
		/**@brief Assigned cash register.
		 */
		Register& lane;
		
	public:
		/**@brief Mutex signalling the presence of the bill.
		 */
		odemx::synchronization::Res bill;
		
		/**@brief Mutex signalling the presence of payment.
		 */
		odemx::synchronization::Res money;
		
		/**@brief Statistics object used to keep track of the time spent
		 * waiting for new customers.
		 */
		odemx::statistics::Sum idleTime;
		
		/**@brief Statistics object used to keep track of the sales.
		 */
		odemx::statistics::Sum salesVol;
		
		/**@brief Constructor for cashiers.
		 * @param[in] sim   the simulation
		 * @param[in] lane  
		 */
		Cashier(Simulation& sim, Register& lane);
		
		/**@brief Calculates the costs of all products for each
		 * customer, transfers the merchandise from the belt to the tray
		 * and receives the payment.
		 */
		virtual int main();
	};
}

#endif /* CASHIER_HPP_INCLUDED */
