/***************************************************************************//**
 * @file Customer.cpp
 * @author Dorian Weber
 * @brief Contains the implementation of the Customer process.
 ******************************************************************************/

#include "Customer.hpp"

Checkout::Register& Checkout::Customer::selectLane() {
	Simulation& sim = static_cast<Simulation&>(getSimulation());
	Register* best = nullptr;
	unsigned int itemCount = -1;
	unsigned int custCount = -1;

	// select the register with the shortest line of customers
	for (Register* reg : sim.lanes) {
		if (itemCount > reg->itemCount()
			|| (itemCount == reg->itemCount()
				&& custCount > reg->customerCount())) {
			itemCount = reg->itemCount();
			custCount = reg->customerCount();
			best = reg;
		}
	}

	return *best;
}

int Checkout::Customer::main() {
	Simulation& sim = static_cast<Simulation&>(getSimulation());
	Register& lane = selectLane();
	odemx::base::SimTime delta;
	odemx::base::SimTime start;

	start = getTime();

	// move the products from the cart to the conveyor belt
	lane.aquireBelt();
	delta = getTime();
	for (std::unique_ptr<Merchandise>& product : cart) {
		holdFor(product->duration);
		lane.beltPut(product.release());
	}
	delta = getTime() - delta;
	lane.releaseBelt();

	// collect the merchandise
	lane.aquireTray();
	delta = getTime();
	for (unsigned int i = 0; i < cart.size(); ++i) {
		std::unique_ptr<Merchandise> product = lane.trayGet();
		holdFor(product->duration);
	}
	delta = getTime() - delta;
	lane.releaseTray();

	// wait for the bill to be produced
	lane.cashier.bill.acquire(1);

	// pay the bill
	holdFor(payTime);
	lane.cashier.money.release(1);

	sim.handlingTime.update(getTime() - start);

	// done
	return 0;
}

