/***************************************************************************//**
 * @file CustomerGenerator.cpp
 * @author Dorian Weber
 * @brief Contains the implementation of the CustomerGenerator process.
 ******************************************************************************/

#include "Simulation.hpp"

Checkout::CustomerGenerator::CustomerGenerator(
	Simulation& sim,
	unsigned int productMean,
	double arrivalRate,
	double costMean,
	double weightMean,
	double weightDeviation,
	double payTimeMean,
	double payTimeDeviation,
	unsigned int simDuration
):	Process(sim, "CustomerGenerator"),
	arrivalDist(sim, "ArrivalDistribution", arrivalRate),
	productDist(sim, "ProductCountDistribution", productMean),
	costDist(sim, "CostDistribution", costMean),
	weightDist(sim, "WeightDistribution", weightMean, weightDeviation),
	payTimeDist(sim, "PayTimeDistribution", payTimeMean, payTimeDeviation),
	customers(sim, "CustomerCount"),
	simDuration(simDuration) {
	sim.report.addReportProducer(customers);
}

int Checkout::CustomerGenerator::main() {
	Customer* customer;
	unsigned int i;

	while (getTime() < simDuration) {
		// wait for the next customer to arrive
		holdFor(arrivalDist.sample());

		// create the customer
		{
			double paytime;
			while ((paytime = payTimeDist.sample()) < 0);
			customer = new Customer(static_cast<Simulation&>(getSimulation()), paytime);
		}

		// generate merchandise for the customer
		unsigned int productCount;
		while ((productCount = productDist.sample()) <= 0);

		for (i = productCount; i > 0; --i) {
			double weight, cost;

			while ((weight = weightDist.sample()) < 0.0);
			while ((cost = costDist.sample()) < 0.0);

			customer->productAdd(new Merchandise(weight, cost));
		}

		// go!
		customers.update();
		customer->activate();
	}

	return 0;
}
