/***************************************************************************//**
 * @file Merchandise.hpp
 * @author Dorian Weber
 * @brief Contains a passive structure for the Merchandise.
 ******************************************************************************/

#ifndef MERCHANDISE_HPP_INCLUDED
#define MERCHANDISE_HPP_INCLUDED

#include "Simulation.hpp"

namespace Checkout {
	/**@brief Passive structure for merchandise to be moved by Customers
	 * and Cashiers.
	 */
	struct Merchandise: odemx::synchronization::PortData {
		odemx::base::SimTime duration; ///<@brief The time to move it.
		double price;                  ///<@brief The price tag.

		/**@brief Constructor for merchandise.
		 * @param[in] duration  the time needed to move it
		 * @param[in] price     the price tag
		 */
		Merchandise(odemx::base::SimTime duration, double price)
		:	duration(duration),
			price(price) {}
	};
}

#endif /* MERCHANDISE_HPP_INCLUDED */
