/***************************************************************************//**
 * @file Simulation.cpp
 * @author Dorian Weber
 * @brief Implementation of the Checkout scenario simulation context.
 ******************************************************************************/

#include "Simulation.hpp"

Checkout::Simulation::Simulation(
	unsigned int duration,
	unsigned int registerCount,
	unsigned int beltCap,
	unsigned int trayCap,
	unsigned int productMean,
	double arrivalRate,
	double costMean,
	double weightMean,
	double weightDeviation,
	double paytimeMean,
	double paytimeDeviation,
	bool defaultReport
):	odemx::base::Simulation("CheckoutSimulation"),
	report(std::cout),
	arrivals(*this, productMean, arrivalRate, costMean, weightMean,
		weightDeviation, paytimeMean, paytimeDeviation, duration),
	handlingTime(*this, "HandlingTime", 10, 200, 19),
	idleTime(*this, "IdleTimeTotal"),
	salesVol(*this, "SalesVolumeTotal") {
	using odemx::data::buffer::StatisticsBuffer;
	if (defaultReport) {
		std::shared_ptr<StatisticsBuffer> stats = StatisticsBuffer::create(*this, "Buffer");
		report.addReportProducer(*stats);
		addConsumer(stats);
	}

	report.addReportProducer(handlingTime);
	report.addReportProducer(idleTime);
	report.addReportProducer(salesVol);

	for (unsigned int i = 0; i < registerCount; ++i)
		lanes.push_back(new Register(*this, beltCap, trayCap));
}

void Checkout::Simulation::initSimulation() {
	arrivals.activate();

	for (Register *reg: lanes)
		reg->cashier.activate();
}
