use crate::RandomVariable;
use uom::si::{f64::Time, time};

#[derive(Debug)]
#[repr(C)]
struct Stats {
	ferry_cargo_len: RandomVariable,
	ferry_load_time: RandomVariable,
	car_wait_time: RandomVariable,
}

mod ffi {
	use std::os::raw::{c_double, c_uint};

	#[link(name = "odemx-examples", kind = "static")]
	extern "C" {
		pub fn ferry(duration: c_double, ferries: c_uint, harbors: c_uint) -> super::Stats;
	}
}

pub fn ferry(duration: Time, ferries: usize, harbors: usize) -> impl std::fmt::Debug {
	unsafe {
		ffi::ferry(
			duration.get::<time::minute>() as _,
			ferries as _,
			harbors as _,
		)
	}
}
