# ODΞM-rs Library

## Overview

ODΞM-rs is a comprehensive object-oriented discrete-event modeling library for Rust, leveraging Rust's async/await syntax for defining and executing concurrent processes in simulation environments. This library serves as the foundation for building detailed and complex simulation models, providing a cohesive integration of various specialized modules.

## Key Features

- **Process Modeling:** Enable the definition of asynchronous processes with isolated states, representing actors within a simulation.
- **Discrete-Event Simulation:** Facilitate modeling of events occurring at distinct points in time, allowing accurate representation of the evolution of complex systems.
- **Efficient Execution:** Utilize Rust's async/await syntax for defining concurrent processes, allowing for a high level of dynamism in simulations.
- **Platform Independence:** The library is designed to be `no_std` and in large parts no `alloc` compatible, allowing the exection of simulation models on any device.

## Modules

- **`odem-rs-core`**: Provides the core functionalities for discrete-event modeling and concurrent systems.
- **`odem-rs-meta`**: Offers boilerplate-reducing capabilities using procedural macros to improve the conciseness and readability of the library's functionality.
- **`odem-rs-stat`**: Contains statistical tools for data aggregation and analysis in simulations.
- **`odem-rs-sync`**: Focuses on synchronization and communication structures for managing interactions between simulation entities.
- **`odem-rs-util`**: Offers utility tools for object pool management and random number generation, tailored for simulation scenarios.

## Usage

ODΞM-rs is designed to be both powerful and user-friendly, catering to both beginners and experienced users in the field of simulation modeling. Its modular structure allows for flexibility and scalability, enabling users to incorporate only the components they need for their specific simulation projects.

## Contributing

Contributions to the ODΞM-rs library are highly valued. Whether it is through enhancing existing modules, adding new features, or improving documentation, your input is crucial in making this library more robust and versatile.

## Acknowledgements

The ODΞM-rs library is inspired by advanced concepts in discrete-event simulation, async programming, and fiber-based concurrency. It is an ongoing project, with a focus on community involvement and continuous improvement to provide a comprehensive framework for simulation modeling in Rust.
