/***************************************************************************//**
 * @file main.cpp
 * @author Dorian Weber
 * @brief Contains the entry point for the simulator.
 ******************************************************************************/

#include "Simulation.hpp"

#ifdef RUST_FFI
///@brief Entry point for the function used to benchmark this scenario.
extern "C" void checkout(unsigned int duration) {
	Checkout::Simulation sim(
		/* .duration 		 = */  duration,
		/* .registerCount    = */  2,      // number of parallel lanes
		/* .beltCap          = */ 20,      // products per lane
		/* .trayCap          = */ -1,      // space in tray
		/* .productMean      = */  3,      // products per customer
		/* .arrivalRate      = */  1/30.0, // how many customers per sec
		/* .costMean         = */  1/5.0,  // average price
		/* .weightMean       = */  3.0,    // time to transfer merchandise
		/* .weightDeviation  = */  2.0,    // deviation on transfer
		/* .paytimeMean      = */ 10.0,    // time to complete payment
		/* .paytimeDeviation = */  5.0,    // deviation on paytime
		/* .defaultReport    = */ false    // simulation statistics flag
	);
	
	sim.run();
	
	// no report
}
#else
int main()
{
	Checkout::Simulation sim(
		/* .duration 		 = */ 16 /*h*/ * 60 /*m*/ * 60 /*s*/ ,
		/* .registerCount    = */  2,      // number of parallel lanes
		/* .beltCap          = */ 20,      // products per lane
		/* .trayCap          = */ -1,      // space in tray
		/* .productMean      = */  3,      // products per customer
		/* .arrivalRate      = */  1/30.0, // how many customers per sec
		/* .costMean         = */  1/5.0,  // average price
		/* .weightMean       = */  3.0,    // time to transfer merchandise
		/* .weightDeviation  = */  2.0,    // deviation on transfer
		/* .paytimeMean      = */ 10.0,    // time to complete payment
		/* .paytimeDeviation = */  5.0,    // deviation on paytime
		/* .defaultReport    = */ false    // simulation statistics flag
	);
	
	sim.run();
	sim.report.generateReport();
	
	return 0;
}
#endif
