use core::ops::Add;
use odem_rs::{
	core::{config::DefaultConfig, Active},
	prelude::*,
};
use tracing::debug;

async fn arrivals<C, D, T, G, A>(sim: &Sim<C>, mut delay: D, mut gen: G)
where
	C: Config,
	D: FnMut() -> T,
	T: Into<C::Time>,
	G: FnMut() -> A,
	C::Time: Add<Output = C::Time>,
	A: Active<C>,
{
	let pool = pool!();

	loop {
		sim.advance(delay()).await;
		sim.activate(pool.alloc(gen()));
	}
}

#[derive(Debug)]
struct Customer;

impl Customer {
	fn process() -> Process<impl Actions<Config = DefaultConfig>> {
		Process::new(Customer, Self::main)
	}

	async fn main(&self, sim: &Sim) {
		sim.advance(3.0).await;
		debug!("{:?}: tick", self as *const _);
	}
}

async fn sim_main(sim: &Sim) {
	sim.fork(sim.advance(30.0))
		.or(arrivals(sim, || 1.0, Customer::process))
		.await;
	debug!("done");
}

fn main() {
	tracing_subscriber::fmt()
		.with_max_level(tracing::Level::DEBUG)
		.with_target(false)
		.with_timer(model_time!("[{time:2}]"))
		// .without_time()
		.init();

	match Simulator::default().run(sim_main) {
		Ok(_) => {}
		Err(err) => println!("{}", err),
	}
}
