//*****************************************************************************
//      slxdll.h - include file for C, C++ SLX DLLs
//*****************************************************************************

#ifndef SLXDLL_INCLUDED
#define SLXDLL_INCLUDED ONCE_ONLY

#undef CDECL
#undef EXPORT

#define CDECL   __cdecl
#define EXPORT  __declspec(dllexport)

#ifndef EXTERN_C
	#ifdef __cplusplus
		#define EXTERN_C extern "C"
	#else
		#define EXTERN_C
	#endif
#endif

#define SLXBOOL int

#ifndef FALSE
	#define FALSE   0
#endif

#ifndef TRUE
	#define TRUE    1
#endif

#ifndef NONE
	#define NONE    0
#endif

#ifndef NULL
	#define NULL    0L
#endif

typedef enum { CPT_CHECKPOINT, CPT_RESTORE, CPT_RELEASE } CHECKPOINT_TYPE;

struct SLXTransferVector
{
	void (CDECL *ChangeControlVariable)       (void *);
	void (CDECL *ChangePointer)               (void **pointer, void *value);
	void (CDECL *PlaceInto)                   (struct slx_set *, struct object_header *);
	void (CDECL *RemoveFrom)                  (struct slx_set *, struct object_header *);
	struct object_header * (CDECL *SuccInSet) (struct slx_set *, struct object_header *);
	struct object_header * (CDECL *PredInSet) (struct slx_set *, struct object_header *);

	int  (CDECL *BuildQueryFromName)          (char *name,             struct SLXquery *result);
	int  (CDECL *BuildQueryFromPointer)       (void *pointer,          struct SLXquery *result);
	int  (CDECL *QuerySLX)                    (struct SLXquery *query, struct syminfo *result);

	void (CDECL *SLXprint)                    (char *text);
	void (CDECL *SLXStopExecution)            (void);
	int  (CDECL *SLXMessageBox)               (char *title, char *message, int flags);
	void (CDECL *RegisterCheckpointCallback)  (void (CDECL *function) (CHECKPOINT_TYPE ctype, int index));
	void (CDECL *ThrowSLXException)			  (int error_code, char *msg);
};

extern struct SLXTransferVector *SLXTV;

#define SLX_BuildQueryFromName          (*(SLXTV -> BuildQueryFromName))
#define SLX_BuildQueryFromPointer       (*(SLXTV -> BuildQueryFromPointer))
#define SLX_Query                       (*(SLXTV -> QuerySLX))
#define SLX_ChangeControlVariable       (*(SLXTV -> ChangeControlVariable))
#define SLX_ChangePointer               (*(SLXTV -> ChangePointer))
#define SLX_MessageBox                  (*(SLXTV -> SLXMessageBox))
#define SLX_PlaceIntoSet                (*(SLXTV -> PlaceInto))
#define SLX_PredecessorInSet            (*(SLXTV -> PredInSet))
#define SLX_Print                       (*(SLXTV -> SLXprint))
#define SLX_RemoveFromSet               (*(SLXTV -> RemoveFrom))
#define SLX_StopExecution               (*(SLXTV -> SLXStopExecution))
#define SLX_SuccessorInSet              (*(SLXTV -> SuccInSet))
#define SLX_RegisterCheckpointCallback  (*(SLXTV -> RegisterCheckpointCallback))
#define SLX_Throw						(*(SLXTV -> ThrowSLXException))

//*****************************************************************************
//      Starter Library Function Prototypes
//*****************************************************************************

int  CDECL SLX_GetString         (struct string_header *s, char *target, int length);
void CDECL SLX_SetString         (struct string_header *s, char *value);

struct object_header * CDECL SLX_FirstInSet (struct slx_set *s);
struct object_header * CDECL SLX_LastInSet  (struct slx_set *s);

//*****************************************************************************
//      SLX Strings
//*****************************************************************************

struct string_header    // String Descriptor (string constants/variables)
{
	char    *string_address;                // Pointer to actual text
	short   string_current_length,
	        string_max_length;
};

//*****************************************************************************
//      SLX Objects
//*****************************************************************************

struct object_header    // The prefix for all SLX objects
{
	int             use_count;
#ifdef _WIN64
	int             private1;
#endif
	int             objectID,		// ID number for this A/R
	                last_puck_ID;	// most recently used puck ID

	void            *private2[7];	// internal pointers
};

//*****************************************************************************
//      SLX Arrays
//*****************************************************************************

struct array_header
{
	char    *array_data,                    // Address of data
	        *private1;

	short int	array_ndim,                     // # dimensions
	        array_flags;
#define VDIM	1

	int         array_elt_count;                // # elements

	//      followed by N array_dimension structs

};

struct array_dimension
{
	int     dim_lower_bound,                // lower bound of this dimension
	        dim_nelts_m1,                   // number of elts - 1
	        dim_multiplier;                 // bytes between successive indices
};

//*****************************************************************************
//      SLX Control Variables
//*****************************************************************************

#ifdef _WIN64
	#pragma pack(push,4)
#endif

struct control_boolean
{
	void    *private1,
	        *private2;

	int     bvalue;                 // 0 = FALSE, 1 = TRUE
};

struct control_double
{
	void    *private1,
	        *private2;

	double  fvalue;
};

struct control_enum
{
	void    *private1,
	        *private2;

	int     evalue;                 // 0 = NONE, 1...N
};

struct control_int
{
	void    *private1,
	        *private2;

	int     ivalue;
};

struct control_pointer
{
	void    *private1,
	        *private2;

	void    *pvalue;                        // The actual pointer
};

struct control_string
{
	void    *private1,
	        *private2;

	struct string_header    svalue;
};

#ifdef _WIN64
	#pragma pack(pop)
#endif

//*****************************************************************************
//      SLX Sets
//*****************************************************************************

struct set_ecb          // Set element control block
{
#define ECB_POOL_SUCC   ecb_hash_succ           /* The first dword in an ECB */

	struct set_ecb  *ecb_hash_succ,         // hash table successor
		       *ecb_succ_in_set,       /* Successor within this set */
		       *ecb_pred_in_set;       /* Predecessor */

	struct object_header *ecb_object;       /* Pointer to object */
	struct slx_set       *ecb_set;          /* Pointer to set */
	void                 *private1;
};

struct slx_set          // SLX "set" entity
{
	struct object_header    set_header;     // Standard object header
	void                    *private1;

	struct control_int      set_size;       // Element count
	int                     ls_mode;		// Henriksen's Algorithm traversal mode
	struct set_ecb          *set_first,     // Ptr to ctrl block for first elt
		       *set_last;      // and last
	void                    *private3[7];
};

#endif

