#![allow(dead_code, unused_imports, internal_features)]

use odem_rs::{
	prelude::*,
	sync::{
		channel::shared::{Receiver, Sender},
		Publisher,
	},
	util::uom::*,
};
use rand::Rng;
use rand_distr::Exp;
use std::{
	cell::Cell,
	collections::VecDeque,
	future::IntoFuture,
	pin::{pin, Pin},
};
use tracing::{debug, info, trace, Level};
use tracing_subscriber::fmt::format::FmtSpan;

#[derive(Config, Default)]
struct MyConfig {
	#[time]
	time: Time<f64>,
	stats: RandomVariable
}

struct Foo;

impl Foo {
	fn process() -> impl Active<MyConfig> {
		Process::new(Self, Self::actions)
	}
	
	async fn actions(&self, sim: &Sim<MyConfig>) {
		let share = sim.active().unwrap().label();
		debug!("activated {share}");
		sim.advance(minute(1.0)).await;
	}
}

async fn sim_main(sim: &Sim<MyConfig>) -> Time<f64> {
	debug!("begin");

	let my_foo = unique!(Foo::process());
	sim.activate(my_foo).await;
	sim.global().stats.tabulate(10.0);

	debug!("end");
	sim.now()
}

fn main() {
	tracing_subscriber::fmt()
		.with_max_level(Level::DEBUG)
		.with_target(false)
		.with_timer(model_time!("[{time:#2.3}]"))
		.init();
	
	let sim = Simulator::default();
	let result = sim.run(sim_main);
	println!("model_time: {}", result.unwrap().display());
}
